@@ -1,49 +1,49 @@
-/**
- * Copyright (c) 2004-2007 Michael Terry
- * Copyright (c) 2009 Paul Ivanov
- * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
- * 
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+/*
+
+Copyright (c) 2001-2007 Michael Terry
+Copyright (c) 2009 Paul Ivanov
+Copyright (c) 2013-2014 Arthur Borsboom
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+*/
 
 /* required by socket stuff */
 /* define _GNU_SOURCE here because that makes our sockets work nice
  Unfortunately, we lose portability... */
-#define _GNU_SOURCE	1
-#include <stdio.h>
-#include <unistd.h>
-#include <sys/un.h>
-#include <sys/socket.h>
-#include <sys/select.h>
-
-#include <string.h>
-#include <stdlib.h> /* for exit */
 
 #include "../config.h"
-#include <glib/gi18n.h>
-#include <glib/gstdio.h>
 
-#include "fio.h" /* for fio_get_info_from_file */
-#include "help.h"
-#include "prefix.h"
 #include "xpad-app.h"
+#include "help.h"
 #include "xpad-pad.h"
 #include "xpad-pad-group.h"
 #include "xpad-periodic.h"
 #include "xpad-session-manager.h"
-#include "xpad-settings.h"
 #include "xpad-tray.h"
+#include "xpad-debug.h"
+
+#include <string.h>
+#include <sys/socket.h>
+#include <sys/un.h>
+#include <stdio.h>
+#include <stdlib.h>
+
+#include <glib.h>
+#include <glib/gi18n.h>
+#include <glib/gstdio.h>
 
 /* Seems that some systems (sun-sparc-solaris2.8 at least), need the following three #defines. 
    These were provided by Alan Mizrahi <alan@cesma.usb.ve>.
@@ -60,7 +60,6 @@
 #define SUN_LEN(sunp) ((size_t)((struct sockaddr_un *)0)->sun_path + strlen((sunp)->sun_path))
 #endif
 
-
 static gint xpad_argc;
 static gchar **xpad_argv;
 static gboolean option_nonew;
@@ -77,35 +76,35 @@ static gchar *program_path;
 static gchar *server_filename;
 static gint server_fd;
 static FILE *output;
-static gboolean xpad_translucent = FALSE;
 static XpadPadGroup *pad_group;
 static gint pads_loaded_on_start = 0;
+static XpadSettings *settings;
 
-static gboolean  process_local_args         (gint *argc, gchar **argv[]);
-static gboolean  process_remote_args        (gint *argc, gchar **argv[], gboolean have_gtk);
-
-static gboolean  config_dir_exists          (void);
-static gchar    *make_config_dir            (void);
-static void      register_stock_icons       (void);
-static gint      xpad_app_load_pads         (void);
-static gboolean  xpad_app_quit_if_no_pads   (XpadPadGroup *group);
-static gboolean  xpad_app_first_idle_check  (XpadPadGroup *group);
-static gboolean  xpad_app_pass_args         (void);
-static gboolean  xpad_app_open_proc_file    (void);
+static gboolean		process_local_args          (gint *argc, gchar **argv[]);
+static gboolean		process_remote_args         (gint *argc, gchar **argv[], gboolean have_gtk, XpadSettings *xpad_settings);
 
+static gboolean		config_dir_exists           (void);
+static gchar		*make_config_dir            (void);
+static void			register_stock_icons        (void);
+static gint			xpad_app_load_pads          (void);
+static gboolean		xpad_app_quit_if_no_pads    (XpadPadGroup *group);
+static gboolean		xpad_app_first_idle_check   (XpadPadGroup *group);
+static gboolean		xpad_app_pass_args          (void);
+static gboolean		xpad_app_open_proc_file     (void);
 
 static void
 xpad_app_init (int argc, char **argv)
 {
 	gboolean first_time;
 	gboolean have_gtk;
-	// GdkVisual *visual;
 
-	/* Set up i18n */
+	xpad_debug_init();
+	xpad_debug_message (DEBUG_APP, "Startup");
+
+	/* Set up support different languages */
 #ifdef ENABLE_NLS
-	gtk_set_locale ();
 	bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
-	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 #endif
 
@@ -113,14 +112,14 @@ xpad_app_init (int argc, char **argv)
 	xpad_argc = argc;
 	xpad_argv = argv;
 	output = stdout;
-	
+
 	/* Set up config directory. */
 	first_time = !config_dir_exists ();
 	config_dir = make_config_dir ();
-	
+
 	/* create master socket name */
 	server_filename = g_build_filename (xpad_app_get_config_dir (), "server", NULL);
-	
+
 	if (!have_gtk)
 	{
 		/* We don't have GTK+, but we can still do
@@ -129,84 +128,88 @@ xpad_app_init (int argc, char **argv)
 		process_local_args (&xpad_argc, &xpad_argv);
 		if (!xpad_app_pass_args ())
 		{
-			process_remote_args (&xpad_argc, &xpad_argv, FALSE);
+			process_remote_args (&xpad_argc, &xpad_argv, FALSE, settings);
 			fprintf (output, "%s\n", _("Xpad is a graphical program.  Please run it from your desktop."));
 		}
 		exit (0);
 	}
-	
+
 	g_set_application_name (_("Xpad"));
 	gdk_set_program_class (PACKAGE);
-	
-	/* Set up translucency. */
-/*	visual = gdk_visual_get_best_with_depth (32);
-	if (visual)
-	{
-		GdkColormap *colormap;
-		colormap = gdk_colormap_new (visual, TRUE);
-		gtk_widget_set_default_colormap (colormap);
-		xpad_translucent = TRUE;
-	}*/
-	
+
 	/* Set up program path. */
 	if (xpad_argc > 0)
 		program_path = g_find_program_in_path (xpad_argv[0]);
 	else
 		program_path = NULL;
-	
+
 	process_local_args (&xpad_argc, &xpad_argv);
-	
+
 	if (xpad_app_pass_args ())
 		exit (0);
-	
+
 	/* Race condition here, between calls */
 	xpad_app_open_proc_file ();
-	
+
 	register_stock_icons ();
 	gtk_window_set_default_icon_name (PACKAGE);
-	
+
+	/* Read the Xpad configuration file from disk (if exists) */
+	settings = xpad_settings_new ();
+
+	/* Delay program startup, if user configured it, to wait for example for the loading of the systray. */
+	guint autostart_delay;
+	g_object_get (settings, "autostart-delay", &autostart_delay, NULL);
+
+	if (autostart_delay)
+		sleep(autostart_delay);
+
 	pad_group = xpad_pad_group_new();
-	process_remote_args (&xpad_argc, &xpad_argv, TRUE);
-	
-	xpad_tray_open ();
+	process_remote_args (&xpad_argc, &xpad_argv, TRUE, settings);
+
+	xpad_tray_init (settings);
 	xpad_session_manager_init ();
 
 	/* Initialize Xpad-periodic module */
-	Xpad_periodic_init();
-	Xpad_periodic_set_callback("save-content", (XpadPeriodicFunc) xpad_pad_save_content);
-	Xpad_periodic_set_callback("save-info", (XpadPeriodicFunc) xpad_pad_save_info);
-	
+	xpad_periodic_init ();
+	xpad_periodic_set_callback ("save-content", (XpadPeriodicFunc) xpad_pad_save_content);
+	xpad_periodic_set_callback ("save-info", (XpadPeriodicFunc) xpad_pad_save_info);
+
 	/* load all pads */
 	pads_loaded_on_start = xpad_app_load_pads ();
 	if (pads_loaded_on_start == 0 && !option_new) {
 		if (!option_nonew) {
-			GtkWidget *pad = xpad_pad_new (pad_group);
+			GtkWidget *pad = xpad_pad_new (pad_group, settings);
 			gtk_widget_show (pad);
 		}
 	}
-	
+
+	/* Since all pads have been loaded, reprocess the show/hide/toggle option for all pads */
+	if (have_gtk && (option_show))
+		xpad_pad_group_show_all (pad_group);
+	if (have_gtk && (option_hide))
+		xpad_pad_group_close_all (pad_group);
+	if (have_gtk && option_toggle)
+		xpad_pad_group_toggle_hide (pad_group);
+
 	g_idle_add ((GSourceFunc)xpad_app_first_idle_check, pad_group);
-	
+
 	if (first_time)
 		show_help ();
-	
+
 	g_free (server_filename);
 	server_filename = NULL;
 }
 
-
 gint main (gint argc, gchar **argv)
 {
 	xpad_app_init (argc, argv);
-	
+
 	gtk_main ();
-	
+
 	return 0;
 }
 
-
-
-
 /* parent and secondary may be NULL.
  * Returns when user dismisses error.
  */
@@ -214,37 +217,33 @@ void
 xpad_app_error (GtkWindow *parent, const gchar *primary, const gchar *secondary)
 {
 	GtkWidget *dialog;
-	
+
 	if (!xpad_session_manager_start_interact (TRUE))
 		return;
-	
+
 	g_printerr ("%s\n", primary);
-	
-	dialog = xpad_app_alert_new (parent, GTK_STOCK_DIALOG_ERROR, primary, secondary);
-	gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_OK, 1, NULL);
+
+	dialog = xpad_app_alert_dialog (parent, "dialog-error", primary, secondary);
+	gtk_dialog_add_buttons (GTK_DIALOG (dialog), _("_Ok"), GTK_RESPONSE_OK, NULL);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
-	
+
 	xpad_session_manager_stop_interact (FALSE);
 }
 
-
-
-G_CONST_RETURN gchar *
+const gchar *
 xpad_app_get_config_dir (void)
 {
 	return config_dir;
 }
 
-
 /* Returns absolute path to our own executable. May be NULL. */
-G_CONST_RETURN gchar *
+const gchar *
 xpad_app_get_program_path (void)
 {
 	return program_path;
 }
 
-
 XpadPadGroup *
 xpad_app_get_pad_group (void)
 {
@@ -254,26 +253,24 @@ xpad_app_get_pad_group (void)
 void
 xpad_app_quit (void)
 {
-	// Free the memory used by the pads belonging to this group
-	xpad_pad_group_destroy_pads(xpad_app_get_pad_group());
+	/* Free the memory used by the pads belonging to this group */
+	xpad_pad_group_destroy_pads (pad_group);
 
-	// Free the memory used by group.
-	g_object_unref(xpad_app_get_pad_group());
+	/* Free the memory used by group. */
+	if (G_IS_OBJECT (pad_group))
+		g_object_unref (pad_group);
+	pad_group = NULL;
 
-	// Free the memory used by the settings menu.
-	g_object_unref(XPAD_SETTINGS(xpad_settings()));
+	/* Free the memory used by the tray icon and its menu. */
+	xpad_tray_dispose (settings);
 
-	// Free the memory used by the tray icon and its menu.
-	xpad_tray_close();
+	/* Free the memory used by the settings menu. */
+	if (G_IS_OBJECT (settings))
+		g_object_unref (settings);
+	settings = NULL; /* This is needed due to the asynchronous finalizing process. */
 
-	// Give GTK the signal to clean the rest and quit the application.
-	gtk_main_quit();
-}
-
-gboolean
-xpad_app_get_translucent (void)
-{
-	return xpad_translucent;
+	/* Give GTK the signal to clean the rest and quit the application. */
+	gtk_main_quit ();
 }
 
 static gboolean
@@ -281,11 +278,11 @@ config_dir_exists (void)
 {
 	gchar *dir = NULL;
 	gboolean exists = FALSE;
-	
+
 	dir = g_build_filename (g_get_user_config_dir (), PACKAGE, NULL);
 	exists = g_file_test (dir, G_FILE_TEST_EXISTS);
 	g_free (dir);
-	
+
 	if (!exists)
 	{
 		/* For backwards-compatibility, we see if the old location for
@@ -294,7 +291,7 @@ config_dir_exists (void)
 		exists = g_file_test (dir, G_FILE_TEST_EXISTS);
 		g_free (dir);
 	}
-	
+
 	return exists;
 }
 
@@ -335,84 +332,77 @@ make_config_dir (void)
 	make_path (g_get_user_config_dir ());
 
 	dir = g_build_filename (g_get_user_config_dir (), PACKAGE, NULL);
-	
+
 	if (!g_file_test (dir, G_FILE_TEST_EXISTS))
 	{
 		gchar *olddir;
-		
+
 		/* For backwards-compatibility, we see if the old location for
 		   configuration files exists.  If so, we move it. */
 		olddir = g_build_filename (g_get_home_dir (), "." PACKAGE, NULL);
-		
+
 		if (g_file_test (olddir, G_FILE_TEST_EXISTS))
 			g_rename (olddir, dir);
 		else
 			g_mkdir (dir, 0700); /* give user all rights */
-		
+
 		g_free (olddir);
 	}
-	
+
 	return dir;
 }
 
-
 /**
- * Creates an alert with 'stock' used to create an icon and parent text of 'parent',
+ * Creates an alert with a named-icon used to create an icon and parent text of 'parent',
  * secondary text of 'secondary'.  No buttons are added.
  */
 GtkWidget *
-xpad_app_alert_new (GtkWindow *parent, const gchar *stock,
-                    const gchar *primary, const gchar *secondary)
+xpad_app_alert_dialog (GtkWindow *parent, const gchar *icon_name, const gchar *primary, const gchar *secondary)
 {
 	GtkWidget *dialog, *hbox, *image, *label;
 	gchar *buf;
-	
-	dialog = gtk_dialog_new_with_buttons (
-		"",
-		parent,
-		GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
-		NULL);
-	
-	hbox = gtk_hbox_new (FALSE, 12);
-	image = gtk_image_new_from_stock (stock, GTK_ICON_SIZE_DIALOG);
+
+	dialog = gtk_dialog_new ();
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
+	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_DIALOG);
 	label = gtk_label_new (NULL);
-	
+
 	if (secondary)
 		buf = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s\n</span>\n%s", primary, secondary);
 	else
 		buf = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>", primary);
-	
+
 	gtk_label_set_markup (GTK_LABEL (label), buf);
 	g_free (buf);
-	
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), hbox);
+
+	gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), 12);
+	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), hbox);
 	gtk_container_add (GTK_CONTAINER (hbox), image);
 	gtk_container_add (GTK_CONTAINER (hbox), label);
-	
+
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0);
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 12);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-	
+
 	gtk_widget_show_all (hbox);
-	
+
 	return dialog;
 }
 
-
 static void
 register_stock_icons (void)
 {
-	GtkIconTheme *theme;
-
-	theme = gtk_icon_theme_get_default ();
+	GtkIconTheme *theme = gtk_icon_theme_get_default ();
 	gtk_icon_theme_prepend_search_path (theme, THEME_DIR);
 }
 
-
 static gboolean
 xpad_app_quit_if_no_pads (XpadPadGroup *group)
 {
@@ -427,11 +417,10 @@ xpad_app_quit_if_no_pads (XpadPadGroup *group)
 				exit (0);
 		}
 	}
-	
+
 	return FALSE;
 }
 
-
 static gboolean
 xpad_app_first_idle_check (XpadPadGroup *group)
 {
@@ -454,11 +443,10 @@ xpad_app_first_idle_check (XpadPadGroup *group)
 				exit (0);
 		}
 	}
-	
+
 	return FALSE;
 }
 
-
 static void
 xpad_app_pad_added (XpadPadGroup *group, XpadPad *pad)
 {
@@ -473,25 +461,25 @@ xpad_app_load_pads (void)
 {
 	gint opened = 0;
 	GDir *dir;
-	G_CONST_RETURN gchar *name;
-	
+	const gchar *name;
+
 	g_signal_connect (pad_group, "pad-added", G_CALLBACK (xpad_app_pad_added), NULL);
-	
+
 	dir = g_dir_open (xpad_app_get_config_dir (), 0, NULL);
-	
+
 	if (!dir)
 	{
 		gchar *errtext;
-		
+
 		errtext = g_strdup_printf (_("Could not open directory %s."), xpad_app_get_config_dir ());
-		
+
 		xpad_app_error (NULL, errtext,
 			_("This directory is needed to store preference and pad information.  Xpad will close now."));
 		g_free (errtext);
-		
+
 		exit (1);
 	}
-	
+
 	while ((name = g_dir_read_name (dir)))
 	{
 		/* if it's an info file, but not a backup info file... */
@@ -499,50 +487,49 @@ xpad_app_load_pads (void)
 		    name[strlen (name) - 1] != '~')
 		{
 			gboolean show = TRUE;
-			GtkWidget *pad = xpad_pad_new_with_info (pad_group, name, &show);
+			GtkWidget *pad = xpad_pad_new_with_info (pad_group, settings, name, &show);
 			if ((show || option_show) && !option_hide)
 				gtk_widget_show (pad);
-		  else if (show) /* pad thought it would show, we should save that it didn't */
-		    xpad_pad_save_info (XPAD_PAD (pad));
+		else if (show) /* pad thought it would show, we should save that it didn't */
+			xpad_pad_save_info (XPAD_PAD (pad));
 			
 			opened ++;
 		}
 	}
-	
+
 	g_dir_close (dir);
-	
+
 	return opened;
 }
 
-
-
-
-
-
-
-
-
-
-
-
 /*
 converts main program arguments into one long string.
 puts allocated string in dest, and returns size
 */
-static gint
+static guint
 args_to_string (int argc, char **argv, char **dest)
 {
-	gint i;
-	gint size = 0;
-	gchar *p;
-	
-	for (i = 0; i < argc; i++)
-		size += strlen (argv[i]) + 1;
-	
+	gint i = 0;
+	guint size = 0;
+	gchar *p = NULL;
+	size_t string_length = 0;
+
+	for (i = 0; i < argc; i++) {
+		string_length = strlen (argv[i]) + 1;
+
+		/* safe cast */
+		if( string_length <= UINT_MAX ) {
+		       size += (guint) string_length;
+		}
+		else {
+			g_warning("casting the size of the arguments failed");
+		}
+	}
+
 	*dest = g_malloc (size);
-	
+
 	p = *dest;
-	
+
 	for (i = 0; i < argc; i++)
 	{
 		strcpy (p, argv[i]);
@@ -550,127 +537,146 @@ args_to_string (int argc, char **argv, char **dest)
 		p[0] = ' ';
 		p += 1;
 	}
-	
+
 	p --;
 	p[0] = '\0';
-	
+
 	return size;
 }
 
-
 /*
 returns number of strings in newly allocated argv
 */
-static gint
+static guint
 string_to_args (const char *string, char ***argv)
 {
-	gint num, i;
+	guint num, i;
 	const gchar *tmp;
 	char **list;
-	
+
 	/* first, find out how many arguments we have */
 	num = 1;
 	for (tmp = strchr (string, ' '); tmp; tmp = strchr (tmp+1, ' '))
-	  num++;
-	
+		num++;
+
 	list = (char **) g_malloc (sizeof (char *) * (num + 1));
-	
+
 	for (i = 0; i < num; i++)
 	{
 		size_t len;
-		
+
 		/* string points to beginning of current arg */
 		tmp = strchr (string, ' '); /* NULL or end of this arg */
-		
-		if (tmp) len = tmp - string;
-		else   len = strlen (string);
-		
+
+		if (tmp) {
+			long int difference = tmp - string;
+			/* safe cast from long int to size_t */
+			if (difference >= 0)
+				len = (size_t) difference;
+			else {
+				g_warning("Error casting argument length. Arguments might not be processed correctly.");
+				len = 0;
+			}
+		}
+		else
+			len = strlen (string);
+
 		list[i] = g_malloc (len + 1);
 		strncpy (list[i], string, len);
 		list[i][len] = '\0';
-		
+
 		/* make string point to beginning of next arg */
 		string = tmp + 1;
 	}
-	
-	list[i] = NULL;	/* null terminate list */
-	
+
+	list[i] = NULL;  /* null terminate list */
+
 	*argv = list;
-	
+
 	return num;
 }
 
-#include <errno.h>
 /* This reads a line from the proc file.  This line will contain arguments to process. */
 static void
 xpad_app_read_from_proc_file (void)
 {
-	gint client_fd, size;
+	gint client_fd;
+	guint size = 0;
 	gint argc;
 	gchar **argv;
 	gchar *args;
 	struct sockaddr_un client;
 	socklen_t client_len;
-	size_t bytes;
-	
+	ssize_t bytes = -1;
+
 	/* accept waiting connection */
 	client_len = sizeof (client);
 	client_fd = accept (server_fd, (struct sockaddr *) &client, &client_len);
 	if (client_fd == -1)
 		return;
-	
-	/* get size of args */
+
+	/* get size of args and verify for errors */
 	bytes = read (client_fd, &size, sizeof (size));
-	if (bytes != sizeof(size))
+	if (bytes == -1 || bytes != sizeof(size)) {
+		g_warning("Cannot read proc file correctly");
 		goto close_client_fd;
-	
+	}
+
 	/* alloc memory */
 	args = (gchar *) g_malloc (size);
 	if (!args)
 		goto close_client_fd;
-	
+
 	/* read args */
 	bytes = read (client_fd, args, size);
 	if (bytes < size)
 		goto close_client_fd;
-	
-	argc = string_to_args (args, &argv);
-	
+
+	argc = (gint) string_to_args (args, &argv);
+
 	g_free (args);
-	
+
 	/* here we redirect singleton->priv->output to the socket */
 	output = fdopen (client_fd, "w");
-	
-	if (!process_remote_args (&argc, &argv, TRUE))
+
+	if (!process_remote_args (&argc, &argv, TRUE, settings))
 	{
 		/* if there were no non-local arguments, insert --new as argument */
 		gint c = 2;
-		gchar **v = g_malloc (sizeof (gchar *) * c);
+		gchar **v = NULL;
+		unsigned long int my_size = 0;
+		/* safe cast */
+		my_size = sizeof (gchar *) * (long unsigned) c;
+		v = g_malloc (my_size);
 		v[0] = PACKAGE;
 		v[1] = "--new";
-		
-		process_remote_args (&c, &v, TRUE);
+
+		process_remote_args (&c, &v, TRUE, settings);
 		
 		g_free (v);
 	}
-	
+
 	/* restore standard singleton->priv->output */
 	fclose (output);
 	output = stdout;
-	
+
 	g_strfreev (argv);
 	return;
-	
+
 close_client_fd:
 	close (client_fd);
 }
 
-
 static gboolean
 can_read_from_server_fd (GIOChannel *source, GIOCondition condition, gpointer data)
 {
+	/* A dirty way to silence the compiler for these unused variables. */
+	(void) source;
+	(void) condition;
+	(void) data;
+
 	xpad_app_read_from_proc_file ();
-	
+
 	return TRUE;
 }
 
@@ -679,27 +685,27 @@ xpad_app_open_proc_file (void)
 {
 	GIOChannel *channel;
 	struct sockaddr_un master;
-	
+
 	g_unlink (server_filename);
-	
+
 	/* create the socket */
 	server_fd = socket (PF_LOCAL, SOCK_STREAM, 0);
 	bzero (&master, sizeof (master)); 
 	master.sun_family = AF_LOCAL;
 	strcpy (master.sun_path, server_filename);
-	
+
 	if (bind (server_fd, (struct sockaddr *) &master, SUN_LEN (&master)))
 		return FALSE;
-	
+
 	/* listen for connections */
 	if (listen (server_fd, 5))
 		return FALSE;
-	
+
 	/* set up input loop, waiting for read */
 	channel = g_io_channel_unix_new (server_fd);
 	g_io_add_watch (channel, G_IO_IN, can_read_from_server_fd, NULL);
 	g_io_channel_unref (channel);
-	
+
 	return TRUE;
 }
 
@@ -712,41 +718,41 @@ xpad_app_pass_args (void)
 	fd_set fdset;
 	gchar buf [129];
 	gchar *args = NULL;
-	gint size;
-	gint bytesRead;
+	guint size;
+	ssize_t bytesRead;
 	gboolean connected = FALSE;
-	ssize_t error = NULL;
-	
+	ssize_t error;
+
 	/* create master socket */
 	client_fd = socket (PF_LOCAL, SOCK_STREAM, 0);
 	master.sun_family = AF_LOCAL;
 	strcpy (master.sun_path, server_filename);
-	
+
 	/* connect to master socket */
 	if (connect (client_fd, (struct sockaddr *) &master, SUN_LEN (&master)))
 		goto done;
 	connected = TRUE;
-	
+
 	size = args_to_string (xpad_argc, xpad_argv, &args) + 1;
-	
+
 	/* first, write length of string */
 	error = write (client_fd, &size, sizeof (size));
 	if (error == -1)
 		g_error("There is a problem writing information to the socket.");
-	
+
 	/* now, write string */
 	error = write (client_fd, args, (size_t) size);
 	if (error == -1)
 		g_error("There is a problem writing information to the socket.");
-	
+
 	do
 	{
 		/* wait for response */
 		FD_ZERO (&fdset);
-		FD_SET (client_fd, &fdset);	
+		FD_SET (client_fd, &fdset);
 		/* block until we are answered, or an error occurs */
 		select (client_fd + 1, &fdset, NULL, &fdset, NULL);
-		
+
 		do
 		{
 			bytesRead = read (client_fd, buf, 128);
@@ -765,33 +771,12 @@ xpad_app_pass_args (void)
 	
 done:
 	close (client_fd);
-	
+
 	g_free (args);
-	
+
 	return connected;
 }
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 /**
  * Here are the functions called when arguments are passed to us.
  */
@@ -822,16 +807,16 @@ process_local_args (gint *argc, gchar **argv[])
 	GOptionContext *context;
 	gint argc_copy;
 	gchar **argv_copy;
-	
+
 	option_version = FALSE;
 	option_nonew = FALSE;
-	
+
 	/* We make copies of argc and argv because we actually don't want the 
 	   behavior of g_option_context_parse() that removes entries from the
 	   array. */
 	argc_copy = *argc;
 	argv_copy = g_strdupv (*argv);
-	
+
 	context = g_option_context_new (NULL);
 	g_option_context_add_main_entries (context, local_options, GETTEXT_PACKAGE);
 	/* We do remote here as well, because we want --help to pick them up.  It
@@ -851,19 +836,19 @@ process_local_args (gint *argc, gchar **argv[])
 		fprintf (output, "%s\n", error->message);
 		exit (1);
 	}
-	
+
 	g_option_context_free (context);
-	g_strfreev(argv_copy);
-	
-	return(option_version || option_nonew);
+	g_strfreev (argv_copy);
+
+	return (option_version || option_nonew);
 }
 
 static gboolean
-process_remote_args (gint *argc, gchar **argv[], gboolean have_gtk)
+process_remote_args (gint *argc, gchar **argv[], gboolean have_gtk, XpadSettings *xpad_settings)
 {
 	GError *error = NULL;
 	GOptionContext *context;
-	
+
 	option_new = FALSE;
 	option_files = NULL;
 	option_quit = FALSE;
@@ -871,7 +856,7 @@ process_remote_args (gint *argc, gchar **argv[], gboolean have_gtk)
 	option_hide = FALSE;
 	option_show = FALSE;
 	option_toggle = FALSE;
-	
+
 	context = g_option_context_new (NULL);
 	g_option_context_set_ignore_unknown_options (context, TRUE);
 	g_option_context_set_help_enabled (context, FALSE);
@@ -881,32 +866,35 @@ process_remote_args (gint *argc, gchar **argv[], gboolean have_gtk)
 		if (have_gtk && option_smid)
 			xpad_session_manager_set_id (option_smid);
 		
+		if (!option_new)
+			g_object_get (settings, "autostart-new-pad", &option_new, NULL);
+
 		if (have_gtk && option_new)
 		{
-			GtkWidget *pad = xpad_pad_new (pad_group);
+			GtkWidget *pad = xpad_pad_new (pad_group, settings);
 			gtk_widget_show (pad);
 		}
 
-		if (have_gtk && option_show)
-		  xpad_pad_group_show_all (pad_group);
-		
-		if (have_gtk && option_hide)
-		  xpad_pad_group_close_all (pad_group);
-		
-		if (have_gtk && option_toggle)
-		  xpad_pad_group_toggle_hide (pad_group);
+		if (!option_hide && !option_show) {
+			guint display_pads;
+			g_object_get (xpad_settings, "autostart-display-pads", &display_pads, NULL);
+			if (display_pads == 0)
+				option_show = TRUE;
+			else if (display_pads == 1)
+				option_hide = TRUE;
+		}
 
 		if (have_gtk && option_files)
 		{
 			int i;
 			for (i = 0; option_files[i]; i++)
 			{
-				GtkWidget *pad = xpad_pad_new_from_file (pad_group, option_files[i]);
+				GtkWidget *pad = xpad_pad_new_from_file (pad_group, settings, option_files[i]);
 				if (pad)
 					gtk_widget_show (pad);
 			}
 		}
-		
+
 		if (option_quit)
 		{
 			if (have_gtk && gtk_main_level () > 0)
@@ -921,9 +909,9 @@ process_remote_args (gint *argc, gchar **argv[], gboolean have_gtk)
 		/* Don't quit.  Bad options passed to the main xpad program by other
 		   iterations shouldn't close the main one. */
 	}
-	
+
 	g_option_context_free (context);
-	
+
 	return(option_new || option_quit || option_smid || option_files ||
 	       option_hide || option_show || option_toggle);
 }
