@@ -1,20 +1,25 @@
-/**
- * Copyright (c) 2004-2007 Michael Terry
- * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
- * 
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+/*
+
+Copyright (c) 2001-2007 Michael Terry
+Copyright (c) 2013-2014 Arthur Borsboom
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+*/
+
+#include "../config.h"
 
 #include "xpad-grip-tool-item.h"
 
@@ -25,10 +30,7 @@ struct XpadGripToolItemPrivate
 
 G_DEFINE_TYPE_WITH_PRIVATE(XpadGripToolItem, xpad_grip_tool_item, GTK_TYPE_TOOL_ITEM)
 
-static void xpad_grip_tool_item_dispose (GObject *object);
-static void xpad_grip_tool_item_finalize (GObject *object);
-
-static gboolean xpad_grip_tool_item_event_box_expose (GtkWidget *widget, GdkEventExpose *event);
+static gboolean xpad_grip_tool_item_event_box_draw (GtkWidget *widget, cairo_t *cr);
 static void xpad_grip_tool_item_event_box_realize (GtkWidget *widget);
 static gboolean xpad_grip_tool_item_button_pressed_event (GtkWidget *widget, GdkEventButton *event);
 
@@ -41,10 +43,6 @@ xpad_grip_tool_item_new (void)
 static void
 xpad_grip_tool_item_class_init (XpadGripToolItemClass *klass)
 {
-	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-	
-	gobject_class->dispose = xpad_grip_tool_item_dispose;
-	gobject_class->finalize = xpad_grip_tool_item_finalize;
 }
 
 static void
@@ -52,54 +50,41 @@ xpad_grip_tool_item_init (XpadGripToolItem *grip)
 {
 	GtkWidget *alignment;
 	gboolean right;
-	
+
 	grip->priv = xpad_grip_tool_item_get_instance_private(grip);
-	
+
 	grip->priv->drawbox = gtk_drawing_area_new ();
 	gtk_widget_add_events (grip->priv->drawbox, GDK_BUTTON_PRESS_MASK | GDK_EXPOSURE_MASK);
 	g_signal_connect (grip->priv->drawbox, "button-press-event", G_CALLBACK (xpad_grip_tool_item_button_pressed_event), NULL);
 	g_signal_connect (grip->priv->drawbox, "realize", G_CALLBACK (xpad_grip_tool_item_event_box_realize), NULL);
-	g_signal_connect (grip->priv->drawbox, "expose-event", G_CALLBACK (xpad_grip_tool_item_event_box_expose), NULL);
+	g_signal_connect (grip->priv->drawbox, "draw", G_CALLBACK (xpad_grip_tool_item_event_box_draw), NULL);
 	gtk_widget_set_size_request (grip->priv->drawbox, 18, 18);
-	
-	right =	gtk_widget_get_direction (grip->priv->drawbox) == GTK_TEXT_DIR_LTR;
+
+	right = gtk_widget_get_direction (grip->priv->drawbox) == GTK_TEXT_DIR_LTR;
 	alignment = gtk_alignment_new (right ? 1 : 0, 1, 0, 0);
-	
+
 	gtk_container_add (GTK_CONTAINER (alignment), grip->priv->drawbox);
 	gtk_container_add (GTK_CONTAINER (grip), alignment);
 }
 
-static void
-xpad_grip_tool_item_dispose (GObject *object)
-{
-	G_OBJECT_CLASS (xpad_grip_tool_item_parent_class)->dispose (object);
-}
-
-static void
-xpad_grip_tool_item_finalize (GObject *object)
-{
-	G_OBJECT_CLASS (xpad_grip_tool_item_parent_class)->finalize (object);
-}
-
-
 static gboolean
 xpad_grip_tool_item_button_pressed_event (GtkWidget *widget, GdkEventButton *event)
 {
 	if (event->button == 1)
 	{
 		GdkWindowEdge edge;
-		
+
 		if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
 			edge = GDK_WINDOW_EDGE_SOUTH_EAST;
 		else
 			edge = GDK_WINDOW_EDGE_SOUTH_WEST;
-	
+
 		gtk_window_begin_resize_drag (GTK_WINDOW (gtk_widget_get_toplevel (widget)),
-			edge, event->button, event->x_root, event->y_root, event->time);
-		
+			edge, (gint) event->button, (gint) event->x_root, (gint) event->y_root, event->time);
+
 		return TRUE;
 	}
-	
+
 	return FALSE;
 }
 
@@ -109,37 +94,25 @@ xpad_grip_tool_item_event_box_realize (GtkWidget *widget)
 	GdkDisplay *display = gtk_widget_get_display (widget);
 	GdkCursorType cursor_type;
 	GdkCursor *cursor;
-	
+
 	if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
 		cursor_type = GDK_BOTTOM_RIGHT_CORNER;
 	else
 		cursor_type = GDK_BOTTOM_LEFT_CORNER;
-	
+
 	cursor = gdk_cursor_new_for_display (display, cursor_type);
-	gdk_window_set_cursor (widget->window, cursor);
-	gdk_cursor_unref (cursor);
+	gdk_window_set_cursor (gtk_widget_get_window(widget), cursor);
+	g_object_unref (cursor);
 }
 
 static gboolean
-xpad_grip_tool_item_event_box_expose (GtkWidget *widget, GdkEventExpose *event)
+xpad_grip_tool_item_event_box_draw (GtkWidget *widget, cairo_t *cr)
 {
-	GdkWindowEdge edge;
-	
-	if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
-		edge = GDK_WINDOW_EDGE_SOUTH_EAST;
-	else
-		edge = GDK_WINDOW_EDGE_SOUTH_WEST;
-	
-	gtk_paint_resize_grip (
-		widget->style,
-		widget->window,
-		GTK_WIDGET_STATE (widget),
-		NULL,
-		widget,
-		"xpad-grip-tool-item",
-		edge,
-		0, 0,
-		widget->allocation.width, widget->allocation.height);
+	gtk_render_handle(gtk_widget_get_style_context(widget),
+			cr,
+			0, 0,
+			gtk_widget_get_allocated_width(widget),
+			gtk_widget_get_allocated_width(widget));
 	
 	return FALSE;
 }
