@@ -1,20 +1,23 @@
-/**
- * Copyright (c) 2004-2007 Michael Terry
- * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
- * 
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+/*
+
+Copyright (c) 2001-2007 Michael Terry
+Copyright (c) 2013-2014 Arthur Borsboom
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+*/
 
 #ifndef __XPAD_PAD_GROUP_H__
 #define __XPAD_PAD_GROUP_H__
@@ -37,8 +40,6 @@ typedef struct XpadPadGroupPrivate XpadPadGroupPrivate;
 struct XpadPadGroup
 {
 	GObject parent;
-	
-	/*< private >*/
 	XpadPadGroupPrivate *priv;
 };
 
@@ -46,9 +47,7 @@ struct XpadPadGroupClass
 {
 	GObjectClass parent_class;
 	
-	/* Signals */
-	void (* pad_added)      (XpadPadGroup *group,
-	                         GtkWidget *pad);
+	void (* pad_added)      (XpadPadGroup *group, GtkWidget *pad);
 };
 
 GType    xpad_pad_group_get_type (void);
@@ -62,8 +61,8 @@ void     xpad_pad_group_close_all        (XpadPadGroup *group);
 void     xpad_pad_group_show_all         (XpadPadGroup *group);
 void     xpad_pad_group_toggle_hide      (XpadPadGroup *group);
 GSList * xpad_pad_group_get_pads         (XpadPadGroup *group);
-gint     xpad_pad_group_num_visible_pads (XpadPadGroup *group);
-void     xpad_pad_group_save_unsaved_all (XpadPadGroup *group);
+guint    xpad_pad_group_num_pads         (XpadPadGroup *group);
+guint    xpad_pad_group_num_visible_pads (XpadPadGroup *group);
 void     xpad_pad_group_destroy_pads     (XpadPadGroup *group);
 
 G_END_DECLS
