@@ -3,6 +3,7 @@
 Copyright (c) 2001-2007 Michael Terry
 Copyright (c) 2009 Paul Ivanov
 Copyright (c) 2011 Sergei Riaguzov
+Copyright (c) 2013-2014 Arthur Borsboom
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,8 +24,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #ifndef __XPAD_PAD_H__
 #define __XPAD_PAD_H__
 
-#include <gtk/gtk.h>
 #include "xpad-pad-group.h"
+#include "xpad-settings.h"
 
 G_BEGIN_DECLS
 
@@ -41,7 +42,6 @@ typedef struct XpadPad XpadPad;
 
 struct XpadPad
 {
-   /* private */
    GtkWindow parent;
    XpadPadPrivate *priv;
 };
@@ -55,9 +55,9 @@ struct XpadPadClass
 
 GType xpad_pad_get_type (void);
 
-GtkWidget *xpad_pad_new (XpadPadGroup *group);
-GtkWidget *xpad_pad_new_with_info (XpadPadGroup *group, const gchar *info_filename, gboolean *show);
-GtkWidget *xpad_pad_new_from_file (XpadPadGroup *group, const gchar *filename);
+GtkWidget *xpad_pad_new (XpadPadGroup *group, XpadSettings *settings);
+GtkWidget *xpad_pad_new_with_info (XpadPadGroup *group, XpadSettings *settings, const gchar *info_filename, gboolean *show);
+GtkWidget *xpad_pad_new_from_file (XpadPadGroup *group, XpadSettings *settings, const gchar *filename);
 void xpad_pad_close (XpadPad *pad);
 void xpad_pad_toggle (XpadPad *pad);
 void xpad_pad_save_info (XpadPad *pad);
@@ -72,6 +72,8 @@ void xpad_pad_notify_has_selection (XpadPad *pad);
 void xpad_pad_notify_clipboard_owner_changed (XpadPad *pad);
 void xpad_pad_notify_undo_redo_changed (XpadPad *pad);
 
+void xpad_pad_append_pad_titles_to_menu (GtkWidget *menu);
+
 G_END_DECLS
 
 #endif /* __XPAD_PAD_H__ */
