@@ -1,27 +1,31 @@
-/**
- * Copyright (c) 2004-2007 Michael Terry
- * Copyright (c) 2011 Sergei Riaguzov
- * 
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
- * 
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+/*
+
+Copyright (c) 2001-2007 Michael Terry
+Copyright (c) 2011 Sergei Riaguzov
+Copyright (c) 2013-2014 Arthur Borsboom
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+*/
 
 #ifndef __XPAD_TEXT_BUFFER_H__
 #define __XPAD_TEXT_BUFFER_H__
 
-#include <gtk/gtk.h>
 #include "xpad-pad.h"
+#include <gtksourceview/gtksource.h>
+#include <gtksourceview/gtksourcebuffer.h>
 
 G_BEGIN_DECLS
 
@@ -38,20 +42,18 @@ typedef struct XpadTextBuffer XpadTextBuffer;
 
 struct XpadTextBuffer
 {
-	GtkTextBuffer parent;
-
-	/* private */
+	GtkSourceBuffer parent;
 	XpadTextBufferPrivate *priv;
 };
 
 struct XpadTextBufferClass
 {
-	GtkTextBufferClass parent_class;
+	GtkSourceBufferClass parent_class;
 };
 
 GType xpad_text_buffer_get_type (void);
 
-XpadTextBuffer *xpad_text_buffer_new (void);
+XpadTextBuffer *xpad_text_buffer_new (XpadPad *pad);
 
 void xpad_text_buffer_set_text_with_tags (XpadTextBuffer *buffer, const gchar *text);
 gchar *xpad_text_buffer_get_text_with_tags (XpadTextBuffer *buffer);
@@ -67,9 +69,6 @@ void xpad_text_buffer_redo (XpadTextBuffer *buffer);
 void xpad_text_buffer_freeze_undo (XpadTextBuffer *buffer);
 void xpad_text_buffer_thaw_undo (XpadTextBuffer *buffer);
 
-XpadPad *xpad_text_buffer_get_pad (XpadTextBuffer *buffer);
-void xpad_text_buffer_set_pad (XpadTextBuffer *buffer, XpadPad *pad);
-
 G_END_DECLS
 
 #endif /* __XPAD_TEXT_BUFFER_H__ */
